#!/usr/bin/env python
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/calc_energy_breakdown_from_results.py,v 1.2.2.1 2015/08/26 22:44:52 sanner Exp $
#

wt_dict = {}
wt_dict['gauss 1'] = -0.035579
wt_dict['gauss 2'] = -0.005156
wt_dict['repulsion'] = 0.840245
wt_dict['hydrophobic'] = -0.035069
wt_dict['Hydrogen'] = -0.587439
wt_dict['rot'] = 0.05846
wt_dict['num_tors_div'] = 1.923

verbose = False
vina_score_only_result_file = None
vina_num_torsions = 0
intramolecular_energy = 0

if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        print "Usage: calc_energy_breakdown_from_vina_output.py -f vina_score_only_result.txt"
        print
        print "    Description of command..."
        print "         -f     vina_score_only_result.txt "
        print "         -n     number of torsions allowed by vina "
        print "                (vina disallows bonds rotating hydroxyls) "
        print "         -i     intramolecular energy "
        print "    Optional parameters:"
        print "        [-v]    verbose output"
        print "        [-o energy_results ('.txt')] (default output filename is energy_results.txt)"

    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'vf:n:i:oh')
        print "in calc_energy_breakdown_from_vina_output.py"      
    except getopt.GetoptError, msg:
        print 'calc_energy_breakdown_from_vina_output.py: %s' %msg
        usage()
        #sys.exit()

    #vina_score_only_result_file = "result_file_vina_score_only.txt"
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('v', '--v'):
            verbose = True
            if verbose: print 'verbose=', verbose
        if o in ('-f', '--f'):
            vina_score_only_result_file = a
            if verbose: print 'set vina_score_only_result_file to ', a
        if o in ('-n', '--n'):
            vina_num_torsions = int(a)
            if verbose: print 'set vina_num_torsions to ', a
        if o in ('-i', '--i'):
            intramolecular_energy = float(a)
        if o in ('o', '--o'):
            output_filename = a
            if verbose: print 'set energy_results file to ', a

    if not vina_score_only_result_file:
        print 'calc_energy_breakdown_from_vina_output.py: vina_score_only_result_file must be specified.'
        usage()
        sys.exit()

    fptr = open(vina_score_only_result_file, 'r')
    lines = fptr.readlines()
    all_models = []
    num_models = 0
    for l in lines:
        if l.find("MODEL ")>-1:
            cur_model = {}               
            all_models.append(cur_model) #@@
            print "processing model %d:" %(all_models.index(cur_model)+1)
            num_models=len(all_models) 
        elif l.find("REMARK VINA RESULT:")>-1:  
            cur_model['energy'] = float(l.split()[3])   #-10.1
            print "    energy = ", cur_model['energy']
        elif l.find('Affinity:')==0:
            cur_model['Affinity'] = float(l.split()[1]) #-10.29562
            print "    Affinity: ", cur_model['Affinity']
        elif l.find("gauss 1")>-1:
            cur_model['gauss 1'] = float(l.split()[-1]) #119.70294
            print "    gauss 1: ", cur_model['gauss 1']
        elif l.find("gauss 2")>-1:
            cur_model['gauss 2'] = float(l.split()[-1]) #2325.85768
            print "    gauss 2: ", cur_model['gauss 2']
        elif l.find("repulsion")==4:
            cur_model['repulsion'] = float(l.split()[-1]) #2.42141
            print "    repulsion: ", cur_model['repulsion']
        elif l.find("hydrophobic")==4:
            cur_model['hydrophobic'] = float(l.split()[-1])#78.61094
            print "    hydrophobic: ", cur_model['hydrophobic']
        elif l.find("Hydrogen")==4:
            cur_model['Hydrogen'] = float(l.split()[-1]) #1.75464
            print "    Hydrogen: ", cur_model['Hydrogen']
    num_models = len(all_models)
    for cur_model_index in range(num_models):
        cur_model = all_models[cur_model_index]
        if verbose: print "wt_dict[num_tors_div] = ", wt_dict['num_tors_div']
        if verbose: print "vina_num_torsions = ", vina_num_torsions
        num_tors_div = 1+ wt_dict['num_tors_div']* vina_num_torsions #@@ vina inored 2 of original 14 torsions
        if verbose: print "num_tors_div =", num_tors_div
        print "Model %d calculated scores:" % ( cur_model_index + 1)
        #wt_dict['gauss 1'] = -0.035579
    	print "    weighted gauss 1 = %6.6f " % ( cur_model['gauss 1']*wt_dict['gauss 1']) 
        #wt_dict['gauss 2'] = -0.005156
    	print "    weighted gauss 2 = %6.6f " % ( cur_model['gauss 2']*wt_dict['gauss 2'])
        #wt_dict['repulsion']= 0.840245
    	print "    weighted repulsion = %6.6f " % ( cur_model['repulsion']*wt_dict['repulsion'])
        #wt_dict['hydrophobic'] = -0.035069
    	print "    weighted hydrophobic = %6.6f " % ( cur_model['hydrophobic']*wt_dict['hydrophobic'])
        #wt_dict['Hydrogen'] = 0.0587439
    	print "    weighted Hydrogen = %6.6f " %( cur_model['Hydrogen']*wt_dict['Hydrogen'])
    	score = wt_dict['gauss 1']*cur_model['gauss 1'] + wt_dict['gauss 2']*cur_model['gauss 2'] + wt_dict['repulsion'] *cur_model ['repulsion']+ wt_dict['hydrophobic']*cur_model['hydrophobic'] + wt_dict['Hydrogen']*cur_model['Hydrogen'] 
    	print "    score without num_tors_div for model %d is %6.6f " % ( cur_model_index+1, score)   
        #num_tors_div = 1+ wt_dict['num_tors_div']* vina_num_torsions #@@ vina ignored 2 of original 14 torsions
        #print "    num_tors_div = % 6.6f " % ( num_tors_div )
        w = 0.1 * ( wt_dict['num_tors_div'] + 1)
        print "    w = ", w, " and vina_num_torsions =", vina_num_torsions
        #print "    score/num_tors_div   is %6.6f " % ( score/num_tors_div)
        final_score = score/(1 + w * vina_num_torsions/5.0)
        print "    Final_score (score divided by (1+w *vina_num_torsions/5.0)) is %6.6f " % (final_score)
        # rh empirical: print "    alternative final score (score * 0.568137) is ", score * 0.568137 
    	cur_model['score'] = final_score
    	#print "score for model %d",  cur_model_index+1, " is ", cur_model['score']
        #print "model %d:" %(cur_model_index+1)
    	#print "   Final score is %6.6f " %( cur_model['score'])
        print "    Affinity  is %6.6f " %( cur_model['Affinity']) 
    	#cur_model['score'] = final_score
    
    



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/compute_consensus_maps_from_dlgs.py,v 1.9.2.1 2016/02/11 09:24:08 annao Exp $
#
# $Id: compute_consensus_maps_from_dlgs.py,v 1.9.2.1 2016/02/11 09:24:08 annao Exp $
#
import os, glob, numpy, math
from AutoDockTools.Docking import Docking
from PyAutoDock.AutoGrid import GridMap
from MolKit.molecule import Atom, AtomSet, Bond, Molecule
from MolKit.protein import Protein,Chain,ChainSet, Residue,ResidueSet
from MolKit.pdbWriter import PdbWriter

RT = 0.831*298.
from math import log, e
debug = False

def buildPdb(map_dict, npts, name='DlgBuilt', ctr=0, outputfile='results.pdb', 
                    scale=1.0):
    if debug: print "in buildPdb: tolerance=", tolerance
    name = 'DlgBuilt'
    mol = Protein(name=name)
    mol.curChain = Chain()
    mol.chains = ChainSet([mol.curChain])
    mol.curRes = Residue()
    mol.curChain.adopt(mol.curRes)
    mol.allAtoms = AtomSet()
    mol.curRes.atoms = mol.allAtoms
    nzpts=nypts=nxpts = npts
    #nxpts, nypts, nzpts = npts
    ctr = 0
    for ADtype, m in map_dict.items():
        if debug: 
            print "PROCESSING ", ADtype, " array:", max(m.ravel()), ':', min(m.ravel())
        vals = []
        tctr = 0  #for number of each type
        for z in range(nzpts):
            for y in range(nypts):
                for x in range(nxpts):
                    val = scale * abs(m[x,y,z])
                    vals.append(val)
                    #if abs(val)>.005:
                    if val>tolerance*scale:
                        ctr += 1
                        name = ADtype + str(ctr)
                        #version3:
                        #info_lo = (xcen - numxcells*spacing,
                        #    ycen - numycells*spacing, 
                        #    zcen - numzcells *spacing)
                        #using lower back pt of cube, i think
                        #xcoord = (x-info_lo[0])/spacing
                        #ycoord = (y-info_lo[1])/spacing
                        #zcoord = (z-info_lo[2])/spacing
                        #version2:
                        xcoord = (x-numxcells)*spacing + xcen
                        ycoord = (y-numycells)*spacing + ycen
                        zcoord = (z-numzcells)*spacing + zcen
                        coords = (xcoord,ycoord,zcoord)
                        tctr += 1
                    #    #print "addAtom: name=",name,"ADtype=", ADtype," val=", val, "coords=", coords,"ctr=", ctr
                        addAtom(mol, name, ADtype, val, coords, ctr)
        print "added ",tctr, '<-', ADtype, " atoms"
        if debug:
            print ADtype, ':', tctr , ' ', ctr
    print "total atoms=", ctr
    writer = PdbWriter()
    writer.write(outputfile, mol.allAtoms, records=['ATOM'])


def addAtom(mol, name, ADtype, value, coords, ctr):
    #if debug: print "in addAtom", value,
    res = mol.chains.residues[0]
    chemicalElement = ADtype[0] #???
    childIndex = ctr - 1
    top = mol
    newAt = Atom(name=name, parent=res, top=mol, 
                    chemicalElement=chemicalElement, 
                    childIndex=childIndex)
    newAt.temperatureFactor = value
    newAt.occupancy = value
    newAt.number = ctr
    newAt.conformation = 0
    newAt._coords = [list(coords)]
    newAt.hetatm = 0
    #if debug: print "added ", name, ctr, ':', newAt.full_name(),'-', newAt.parent.children.index(newAt)
    #update allAtoms attribute of this molecule
    mol.allAtoms = mol.chains.residues.atoms


def write_grid_map(filename, spacing, npts, center, score_array,maxval=10000 ):
    if debug:
        print "in write_grid_map ", filename
    stem = os.path.basename(filename).split('.')[0]
    # open and write the file
    fptr = open(filename, 'w')
    # line 1:
    ostr = "GRID_PARAMETER_FILE " + stem + ".gpf\n"
    fptr.write(ostr)
    # line 2:
    ostr = "GRID_DATA_FILE " + stem + ".maps.fld\n"
    fptr.write(ostr)
    # line 3:
    ostr = "MACROMOLECULE " + stem + ".pdbqt\n"
    fptr.write(ostr)
    # line 4:
    ostr = "SPACING " + str(spacing) + "\n"
    fptr.write(ostr)
    # line 5:
    ostr = "NELEMENTS %d %d %d\n" % (npts[0]-1, npts[1]-1, npts[2]-1)
    fptr.write(ostr)
    # line 6:
    ostr = "CENTER %f %f %f\n" % tuple(center)
    fptr.write(ostr)
    # now write the values:
    for z in range(npts[2]):
        for y in range(npts[1]):
            for x in range(npts[0]):
                value = score_array[x,y,z]
                ostr = "%.3f\n" % (value)
                fptr.write(ostr)
    # all done...
    fptr.close()


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: compute_consensus_maps_from_dlgs.py "
        print "    Invoke this script in the directory containing dlg files"
        print "      which are results of multiple dockings to related targets"
        print
        print "    Description of command:"
        print "      Places atoms to build a molecule based on consensus maps "
        print "      which it builds from the results of multiple dockings to related targets"
        print "      By default creates 'results.pdb' based on lowest energy conformations"
        print "                                                 "
        print "    Optional parameters:"
        print "        [-n]    number of pts in each dimension (default is 101)"
        print "        [-s]    spacing between pts (default is 1.0 )"
        print "        [-t]    tolerance (default is .005 )"
        print "        [-o]    output pdb filename"
        print "                (default is 'results.pdb')"
        print "        [-a]    use all conformations  "
        print "                (default is to use only the one with the lowest energy)"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'n:s:t:o:avh')
        if opt_list == []:
            usage()
            sys.exit(2)
    except getopt.GetoptError, msg:
        print 'compute_consensus_maps_from_dlgs.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    # optional parameters
    #-n: number of pts in each dimension 
    num_pts =  101
    #-s: spacing between pts 
    spacing =  1.0
    #-t: energy cutoff for including pt in calculation 
    tolerance =  0.005
    #-o outputfilename
    #-o outputfilename
    outputfilename = "results.pdb"
    #-a  use_all_conformations
    use_all_conformations = True
    #-verbose: chatty output
    verbose = None

    #'n:s:o:avh
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-n', '--n'):
            num_pts = a
            if verbose: print 'set number of pts in each dimension to ', a
        if o in ('-s', '--s'):
            spacing = float(a)
            if verbose: print 'set spacing to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-a', '--a'):
            use_all_conformations = True
            if verbose: print 'set use_all_conformations to True'
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    #read all the docking logs in current directory, one by one
    dlg_list = glob.glob('./*.dlg')
    dockings = []
    #build a list of all atom types in all dlgs
    #it is assumed that all the dockings used the same grids
    ctr = 0
    at_types = {}
    for dlg in dlg_list:
        d = Docking()
        d.readDlg(dlg)
        ctr+= 1
        print ctr, ": read ", dlg
        dockings.append(d)
        for a in d.ligMol.allAtoms:
            at_types[a.autodock_element] = 0
    if debug: print 'at_types=', at_types.keys()
    d = dockings[0]  #get grid info from the first docking
    xcen, ycen, zcen = d.dlo_list[0].parser.center_pt
    #for the output maps...
    # nxgrid=nygrid=nzgrid=npts  ??is this required??
    nxpts = nypts = nzpts = int(num_pts)/2 * 2 + 1 #ensure an odd integer
    npts = (nxpts, nypts, nzpts)
    macroStem = d.dlo_list[0].macroStem
    # build list of all atom types
    all_types  = at_types.keys()
    # setup a map for each atom type
    maps = {}
    norm_maps = {}
    for t in all_types:
        maps[t] = numpy.zeros((nxpts, nypts, nzpts)).astype('f')
        norm_maps[t] = numpy.zeros((nxpts, nypts, nzpts)).astype('f')

    totalEnergy = 0
    #compute the totalEnergy
    if debug: print "computing the totalEnergy"
    N = 0
    for d in dockings:
        len_clusts = len(d.clusterer.clustering_dict.keys())
        if len_clusts:
            if use_all_conformations is True:
                confs = d.ch.conformations
                N += len(confs)
            else:
                key = d.clusterer.clustering_dict.keys()[0]
                confs = [d.clusterer.clustering_dict[key][0][0]]
                N += 1
        else:
            confs = [d.ch.conformations[0]]
        for c in confs:
            d.ch.set_conformation(c)
            deltaG = c.binding_energy
            c.ddG = e**(-deltaG/RT)
            totalEnergy += c.ddG
    #compute the probability of the individual conf._pi = c.ddG/totalEnergy
    print "computing the individual conf._pi"
    for d in dockings:
        len_clusts = len(d.clusterer.clustering_dict.keys())
        #if there is a clustering
        if len_clusts:
            if use_all_conformations is True:
                confs = d.ch.conformations
            else:
                key = d.clusterer.clustering_dict.keys()[0]
                confs = [d.clusterer.clustering_dict[key][0][0]]
        else:
            #if there was no clustering
            if use_all_conformations is True:
                confs = [d.ch.conformations]
            else:
                confs = [d.ch.conformations[0]]
        for c in confs:
            c._pi = c.ddG/totalEnergy
    #compute the probability of finding an atom at a specific pt in the maps
    print "totalEnergy=", totalEnergy
    print "computing the individual atom probabilities"
    for d in dockings:
        len_clusts = len(d.clusterer.clustering_dict.keys())
        if len_clusts:
            if use_all_conformations is True:
                confs = d.ch.conformations
            else:
                key = d.clusterer.clustering_dict.keys()[0]
                confs = [d.clusterer.clustering_dict[key][0][0]]
        else:
            if use_all_conformations is True:
                confs = [d.ch.conformations]
            else:
                confs = [d.ch.conformations[0]]
        for c in confs:
            #update the coordinates
            d.ch.set_conformation(c)
            for atom in d.ligMol.allAtoms:
                #get proper atomic grid
                m = maps[atom.autodock_element]
                #go to nearest pt in grid
                #FIX #1
                x,y,z = atom.coords
                numxcells = nxpts/2  #integer division
                numycells = nypts/2  #integer division
                numzcells = nzpts/2  #integer division
                #6/22: version3
                info_lo = (xcen - numxcells*spacing,
                            ycen - numycells*spacing, 
                            zcen - numzcells *spacing)
                #using lower back pt of cube, i think
                thispt = (int((x-info_lo[0])/spacing),
                          int((y-info_lo[1])/spacing),
                          int((z-info_lo[2])/spacing))
                #4/8:version 2
                #thispt = [  int(x/spacing-xcen+ spacing/2.0 +numxcells), 
                #            int(y/spacing-ycen+ spacing/2.0 +numycells), 
                #            int(z/spacing-zcen+ spacing/2.0 +numzcells)]
                #add this conf's probability to thispt ie: c._pi
                m[thispt] = m[thispt] + c._pi
    #calculate energies based on a Boltzman distribution (~normalizing the maps)
    maxval = -RT * log(tolerance)  #based on tolerance 0.005 cutoff
    if debug: print "tolerance ", tolerance, " yields maxval=", maxval
    minval = 10000
    RTtolerance = -RT*log(tolerance)
    for t in all_types:
        if debug: print "computing ", t, " normalized map"
        for z in range(nzpts):
            for y in range(nypts):
                for x in range(nxpts):
                    value = maps[t][x,y,z]
                    if value < tolerance: #-RTlog(0.005)=1312.06
                        Er = RTtolerance
                        #Er = 0.
                        #.00001
                        #Er = 2851.
                    else:
                        Er = min(-RT*log(value), 10000)
                        if Er<minval:
                            minval = Er
                    norm_maps[t][x,y,z] = Er
    #hd map: max=1290.887.0647.. min=0.0
    #maxval overal=1312.0647... minval=565.784
    # now adjust the values to the typical range of autogrid maps: -2<->2000:
    #maxminval = max(numpy.array(minvals)) #560
    #minminval = min(numpy.array(minvals)) #483
    rng = int(maxval-minval)  #-483
    #rng = int(half_maxval-minminval)  #-483
    if debug: print "maxval=", maxval, '- minval=', minval, ' so rng=', rng
    #print "half_maxval=", half_maxval, '- minminmval=', minminval, ' so rng=', rng
    #TRY1: rng = int(half_maxval-minminval)
    MAPS = {}
    zero_types = []
    for t in all_types:
        #do not process zero-maps
        nmap= norm_maps[t][:]
        #if max(nmap.ravel())!=min(nmap.ravel()):
        if max(nmap.ravel())==min(nmap.ravel()):
            del(norm_maps[t])
            zero_types.append(t) 
            print t, ' is a map of all zeros'
        else:
            nmap = nmap - maxval
            for z in range(nzpts):
                for y in range(nypts):
                    for x in range(nxpts):
                        value = nmap[x,y,z]
                        #if value<10:
                        nmap[x,y,z] = value/rng
                        #nmap[x,y,z] = 2.*value/rng
            MAPS[t] = nmap
    #write out the maps
    center = d.dlo_list[0].parser.center_pt
    if debug: print "writing the grid maps"
    #for atom_type, score_array in norm_maps.items():
    for atom_type, score_array in MAPS.items():
        filename = "%s.%s_combined.map" %(macroStem, atom_type)
        write_grid_map(filename, spacing, npts, center, score_array )
    #if debug: print "building the molecule"
    ####buildPdb(norm_maps, nxpts, name='DlgBuilt', ctr=0, outputfile='results.pdb')
    buildPdb(MAPS, nxpts, name='DlgBuilt', ctr=0, outputfile='results.pdb')

# To execute this command type:
# compute_consensus_maps_from_dlgs.py 



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/prepare_flexreceptor4.py,v 1.7.6.1 2015/08/26 22:45:31 sanner Exp $
#
import os 

from MolKit import Read
from MolKit.protein import ProteinSet, ResidueSet, AtomSet
from MolKit.molecule import BondSet
from MolKit.stringSelector import CompoundStringSelector

from AutoDockTools.MoleculePreparation import AD4FlexibleReceptorPreparation



if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: prepare_flexreceptor4.py -r receptor_filename -s list_of_names_of_residues_to_move"
        print "    Description of command..."
        print "         -r     receptor_filename (.pdbqt)"
        print "         -s     specification for flex residues" 
        print "                Use underscores to separate residue names:"
        print "                  ARG8_ILE84  "
        print "                Use commas to separate 'full names' which uniquely identify residues:"
        print "                  hsg1:A:ARG8_ILE84,hsg1:B:THR4 "
        print "                [syntax is molname:chainid:resname]"
        print "    Optional parameters:"
        print "        [-v]    verbose output"
        print "        [-N]    type(s) of bonds to disallow: "
        print "        [-M]    interactive mode (automatic is default)"
        print "        [-P]    pairs of atom names bonds between which to disallow: hsg1:A:ARG8:CA_CB,CB_CG;hsg1:B:ARG8:CA_CB"
        print "        [-g pdbqt_filename] (rigid output filename)"
        print "        [-x pdbqt_filename] (flexible output filename)"

    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:vs:N:P:g:x:Mh')
    except getopt.GetoptError, msg:
        print 'prepare_flexreceptor4.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-r: receptor
    receptor_filename =  None
    #-s: residues_to_move
    residues_to_move =  None
    # optional parameters
    verbose = None
    #-N: type of bonds to  disallow
    disallow = ""
    #-P: pairs of atom names bonds between which to disallow; residues separated by ';'
    all_disallowed_pairs = ""
    #-g  : rigid output filename
    rigid_filename = None
    #-x  : flexible output filename
    flexres_filename = None
    #-M  : mode
    mode = 'automatic'

    #'r:vs:N:g:x:Mh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-r', '--r'):
            receptor_filename = a
            if verbose: print 'set receptor_filename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to', True
        if o in ('-s', '--s'):
            residues_to_move = a
            if verbose: print 'set residues_to_move to ', a
        if o in ('-N', '--N'):
            disallow = a
            if verbose: print 'set disallow to ', a
        if o in ('-P', '--P'):
            all_disallowed_pairs = a
            if verbose: print 'set all_disallowed_pairs to ', a
        if o in ('-g', '--g'):
            rigid_filename = a
            if verbose: print 'set rigid_filename to ', a
        if o in ('-x', '--x'):
            flexres_filename = a
            if verbose: print 'set flexres_filename to ', a
        if o in ('-M', '--M'):
            mode = 'interactive'
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not  receptor_filename:
        print 'prepare_flexreceptor4: receptor filename must be specified!\n'
        usage()
        sys.exit()

    if not  residues_to_move:
        print 'prepare_flexreceptor4: residues to move must be specified!\n'
        usage()
        sys.exit()

    extension = os.path.splitext(receptor_filename)[1]
    if extension!=".pdbqt":
        print 'prepare_flexreceptor4: receptor file must be in .pdbqt format\n'
        usage()
        sys.exit()

    rec = Read(receptor_filename)[0]
    bnds = rec.buildBondsByDistance()
    if verbose: print 'read ', receptor_filename

    all_res = ResidueSet()
    # hsg1:A:ARG8_ILE82;hsg1:B:THR4 
    # ARG8_ILE84 
    names_by_chain = residues_to_move.split(',')
    if verbose: 
        print "Specified flexres selection strings are:"
        for strN in names_by_chain: 
            print "   %s" %strN
    #1. ['hsg1:A:ARG8_ILE82','hsg1:B:THR4'] 
    # OR 
    #2. ARG8_ILE84
    for res_names_by_chain in names_by_chain:
        #check for ':'
        if verbose: print "selecting ", res_names_by_chain
        if res_names_by_chain.find(':')==-1:
            # no ':' in selection string = simple case eg: ARG8_ILE84 
            res_names = res_names_by_chain.split('_')
            if verbose: print "res_names=", res_names
            res = ResidueSet()
            for n in res_names:
                if verbose: print "looking for  ",n
                matched = rec.chains.residues.get(lambda x: x.name==n)
                if matched.__class__ ==AtomSet:
                    res = matched.parent.uniq() #@@
                if len(res):
                    for residue in res:
                        all_res += res
                        if verbose: print " added ", res.name, " to ", all_res
                else:
                    print "WARNING: no residue named " + n 
        else:
            # '1JFF_protein:A:THR179_GLU183,1JFF_protein:B:TRP21_ARG64_LEU275'
            chainStart = res_names_by_chain.index(':') + 1
            molName = res_names_by_chain[:chainStart-1]
            #if verbose: print "molName = ", molName, " chainStart=", chainStart
            n = res_names_by_chain[chainStart:].replace("_", ",")
            #if verbose: print "after comma replaced '_', n is ", n, '\n'
            selStr = molName + ":" + n
            #if verbose: print "now selStr", selStr
            #eg: 'hsg1:A:ARG8,ILE82'
            result, msg = CompoundStringSelector().select(ProteinSet([rec]), selStr)
            if verbose: print "prepare_flexreceptor4 line 157: result=", result
            if result.__class__!= ResidueSet:
                print residues_to_move, " is not a ResidueSet instead it is a ", result.__class__
            #if verbose: print selStr, " selection =", res, " msg=", msg, '\n'
            resS = ResidueSet()
            if result.__class__ == AtomSet:
                resS = result.parent.uniq()
            else:
                resS = result
            if len(resS):
                all_res += resS
            else:
                print "no residue found using string ", selStr
    #if verbose: print "built all_res=", all_res.full_name()
    #check for duplicates
    d = {}
    for res in all_res: d[res] = 1
    all_res = d.keys()
    all_res = ResidueSet(all_res).uniq()
    all_res.sort()
    if verbose: 
        print "located ", len(all_res),  " residues to format:"
        for z in all_res: print "   %s" %z.full_name()

    all_bnds = BondSet()
    #inactivate bonds between specified atoms:
    #all_disallowed_pairs eg "1g9v_rec:A:ARG532:CA_CB,CB_CG,C_CA;1g9v_rec:B:ARG532:CA_CB,CB_CG"
    if len(all_disallowed_pairs):
        if verbose: print "line 183: all_disallowed_pairs = ", all_disallowed_pairs
        disallowed_pairs = all_disallowed_pairs.split(';') #';' between different residues
        for dp in disallowed_pairs: #1g9v_rec:A:ARG532:CA_CB,CB_CG,C_CA 
            #find the residue
            recN, chainN, resN, res_bnds = dp.split(':') #1g9v_rec, A, ARG532, CA_CB,CB_CB,C_CA
            if recN!=rec.name:
                print dp, " does not match ", rec.name
                exit()
            #result, msg = CompoundStringSelector().select(ProteinSet([rec]), dp)
            if verbose: print "line 199: res_bnds = ", res_bnds
            #r_bnds = res_bnds.split(',')
            #for bnd_pair in res_bnds: # find the residue then process the rest
            #    resK = bnd_pair
            #1g9v_rec:A:ARG532:CA_CB,CB_CG,C_CA   
            #CA_CB, CB_CG          
            #print "r_bnds = ", r_bnds
            for pair in res_bnds.split(','):
                if verbose: print "processing pair ", pair
                names = pair.split('_')
                if verbose: print "names = ", names
                bnds = all_res.atoms.bonds[0].get(lambda x: x.atom1.name in names and x.atom2.name in names)
                if verbose: print "bnds = ", bnds
                if len(bnds):
                    all_bnds += bnds
                    if verbose: 
                        print "line 207: added ",bnds, " to all_bnds"
                        print "all_bnds=", all_bnds
    if verbose:  print "beginning formatting ..."
    fdp = AD4FlexibleReceptorPreparation(rec, residues=all_res, rigid_filename=rigid_filename, 
                                            flexres_filename=flexres_filename,
                                            non_rotatable_bonds=all_bnds, mode = mode)

    if verbose:  print "finished!"

# To execute this command type:
# prepare_flexreceptor4.py -r receptor_filename -s list_of_names_of_residues_to_move"
# eg: in MGLToolsPckgs
#../bin/pythonsh AutoDockTools/Utilities24/prepare_flexreceptor4.py -r hsg1.pdbqt -s hsg1:A:ARG8_ILE84,hsg1:B:THR4 -P "hsg1:A:ARG8:CA_CB" -g hsg1_rigid.pdbqt -x hsg1_flex.pdbqt





